@@ -35,8 +35,7 @@
 #include "apply.h"
 #include "customobject.h"
 #include "localize.h"
-
-static void print_dom(const QDomNode& in, int ind);
+#include "matrix.h"
 
 using namespace Analitza;
 
@@ -95,7 +94,7 @@ Expression::Expression(const Expression & e)
 	: d(new ExpressionPrivate(0))
 {
 	d->m_err=e.d->m_err;
-	if(e.isCorrect())
+	if(e.isCorrect() && e.d->m_tree)
 		d->m_tree = e.d->m_tree->copy();
 }
 
@@ -131,6 +130,11 @@ Expression Expression::operator=(const Expression & e)
 
 bool Expression::setText(const QString & exp)
 {
+	if(exp.isEmpty()) {
+		delete d->m_tree;
+		d->m_tree = 0;
+		return true;
+	}
 	d->m_err.clear();
 	ExpLexer lex(exp);
 	ExpressionParser parser;
@@ -297,7 +301,16 @@ static void variableDepth(Object* o, int& next, const QMap<QString, int>& scope,
 			Vector *v=(Vector*) o;
 			for(Vector::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it)
 				variableDepth(*it, next, scope);
-			
+		}	break;
+		case Object::matrix: {
+			Matrix *v=(Matrix*) o;
+			for(Matrix::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it)
+				variableDepth(*it, next, scope);
+		}	break;
+		case Object::matrixrow: {
+			MatrixRow *v=(MatrixRow*) o;
+			for(MatrixRow::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it)
+				variableDepth(*it, next, scope);
 		}	break;
 		case Object::list: {
 			List *v=(List*) o;
@@ -448,6 +461,12 @@ Object* Expression::ExpressionPrivate::branch(const QDomElement& elem)
 		case Object::list:
 			ret=addElements<List>(new List, &elem);
 			break;
+		case Object::matrix:
+			ret=addElements<Matrix>(new Matrix, &elem);
+			break;
+		case Object::matrixrow:
+			ret=addElements<MatrixRow>(new MatrixRow, &elem);
+			break;
 		case Object::apply: {
 			Apply *a=addElements<Apply>(new Apply, &elem);
 			if(a && !check(a)) {
@@ -503,6 +522,10 @@ enum Object::ObjectType Expression::whatType(const QString& tag)
 		ret= Object::vector;
 	else if(tag=="list")
 		ret= Object::list;
+	else if(tag=="matrix")
+		ret= Object::matrix;
+	else if(tag=="matrixrow")
+		ret= Object::matrixrow;
 	else if(tag=="apply")
 		ret= Object::apply;
 	else if(Operator::toOperatorType(tag)!=0)
@@ -515,7 +538,7 @@ enum Object::ObjectType Expression::whatType(const QString& tag)
 
 bool Expression::operator==(const Expression & e) const
 {
-	return e.d->m_tree && d->m_tree && AnalitzaUtils::equalTree(e.d->m_tree, d->m_tree);
+	return AnalitzaUtils::equalTree(e.d->m_tree, d->m_tree);
 }
 
 bool Expression::operator!=(const Expression& e) const
@@ -532,7 +555,7 @@ void Expression::clear()
 
 bool Expression::isCorrect() const
 {
-	return d && d->m_tree && d->m_err.isEmpty();
+	return !(d || d->m_tree) || d->m_err.isEmpty();
 }
 
 QStringList Expression::bvarList() const
@@ -741,7 +764,18 @@ Expression Expression::elementAt(int position) const
 	
 	return Expression(v->at(position)->copy());
 }
-// 
+
+void Expression::setElementAt(int position, const Expression& exp)
+{
+	Q_ASSERT(isVector());
+	Object* o = actualRoot(d->m_tree);
+	Q_ASSERT(o);
+	Vector *v=static_cast<Vector*>(o);
+	
+	delete v->at(position);
+	v->setAt(position, exp.tree()->copy());
+}
+
 QStringList Expression::error() const
 {
 	return d->m_err;
@@ -813,6 +847,8 @@ static void renameTree(Object* o, int depth, const QString& newName)
 		case Object::container: { CHECK_TYPE(Container); } break;
 		case Object::list: { CHECK_TYPE(List); } break;
 		case Object::vector: { CHECK_TYPE(Vector); } break;
+		case Object::matrix: { CHECK_TYPE(Matrix); } break;
+		case Object::matrixrow: { CHECK_TYPE(MatrixRow); } break;
 		
 		case Object::none:
 		case Object::value:
